<cfcomponent displayname="PandaStreamAPI">
	<cffunction name="sendRequest" access="public" returntype="struct" output="No">
		<cfargument name="setUrl" type="string" required="Yes">
		<cfargument name="setMethod" type="string" required="Yes" default="GET">
		<cfargument name="setRequest" type="string" required="Yes">
		<cfargument name="setParams" type="struct" required="Yes" default="#StructNew()#">
		<cfargument name="setType" type="string" required="Yes" default="FORMFIELD">
		
		<cfset datetime = DateConvert("local2Utc", now())>
		
		<cfhttp url="#setUrl#" method="#setMethod#" timeout="60">
			<cfloop list="#StructKeyList(setParams)#" index="i"><cfhttpparam type="FORMFIELD" name="#i#" value="#setParams[i]#" encoded="yes"></cfloop>
			<cfhttpparam type="#setType#" name="access_key" value="#application.pandastream.access#" encoded="yes">
			<cfhttpparam type="#setType#" name="cloud_id" value="#application.pandastream.cloud#" encoded="yes">
			<cfhttpparam type="#setType#" name="timestamp" value="#dateformat(datetime,'yyyy-mm-dd')#T#TimeFormat(datetime,'HH:mm:ss')#" encoded="yes">
			<cfhttpparam type="#setType#" name="signature" value="#createSignature(setRequest, setMethod, datetime, setParams)#" encoded="yes">
		</cfhttp>
		
		<cfset returnThis = StructNew()>
		
		<cfif (cfhttp.statuscode EQ "200 OK") AND (IsJson(cfhttp.filecontent))>
			<cfset returnThis["status"] = 1>
			<cfset returnThis["data"] = DeserializeJSON(cfhttp.filecontent)>
		<cfelseif (cfhttp.statuscode EQ "200 OK")>
			<cfset returnThis["status"] = 1>
			<cfset returnThis["data"] = cfhttp.filecontent>
		<cfelseif IsJson(cfhttp.filecontent)>
			<cfset pandaError = DeserializeJSON(cfhttp.filecontent)>
			<cfset returnThis["status"] = 0>
			<cfset returnThis["description"] = "#pandaError.error#: #pandaError.message#">
		<cfelse>
			<cfset returnThis["status"] = 0>
			<cfset returnThis["description"] = "#cfhttp.statuscode#">
		</cfif>
		
		<cfreturn returnThis>
	</cffunction>
	
	<cffunction name="createSignature" access="public" returntype="string" output="No">
		<cfargument name="setRequest" type="string" required="Yes">
		<cfargument name="setMethod" type="string" required="Yes">
		<cfargument name="setDatetime" type="date" required="Yes">
		<cfargument name="setParams" type="struct" required="Yes" default="#StructNew()#">
		
		<cfset querystring = "access_key=#encodeRfc3986(application.pandastream.access)#">
		<cfset querystring = ListAppend(querystring, "cloud_id=#encodeRfc3986(application.pandastream.cloud)#", "&")>
		<cfset querystring = ListAppend(querystring,"timestamp=#encodeRfc3986('#dateformat(setDatetime,'yyyy-mm-dd')#T#TimeFormat(setDatetime,'HH:mm:ss')#')#","&")>
		<cfloop list="#StructKeyList(setParams)#" index="i"><cfset querystring = ListAppend(querystring, "#i#=#encodeRfc3986(setParams[i])#", "&")></cfloop>
		<cfset querystring = ListSort(querystring, "textnocase", "ASC", "&")>
		<cfset string_to_sign = "#setMethod##chr(10)##application.pandastream.host##chr(10)#/#setRequest##chr(10)##querystring#">
		
		<cfreturn hmacSHA256(string_to_sign, application.pandastream.secret)>
	</cffunction>
	
	<cffunction name="hmacSHA256" access="private" returntype="string" output="No">
		<cfargument name="yourString" type="string" required="Yes">
		<cfargument name="yourSecret" type="string" required="Yes">
		
		<cfscript>
		secret = createObject('java', 'javax.crypto.spec.SecretKeySpec').Init(yourSecret.GetBytes(), 'HmacSHA256');
		mac = createObject('java', "javax.crypto.Mac");
		mac = mac.getInstance("HmacSHA256");
		mac.init(secret);
		digest = mac.doFinal(yourString.GetBytes());
		signature = ToBase64(digest);
		</cfscript>
		
		<cfreturn signature>
	</cffunction>
	
	<cffunction name="encodeRfc3986" access="private" returntype="string" output="false">
		<cfargument name="stringToEncode" required="yes" type="string">
		
		<!--- Thanks to: http://www.slateblue.org/2009/08/29/url-encoding-with-cfml/ --->
		<cfset netObj = createObject("java","java.net.URLEncoder")>
		<cfset encodedString = netObj.encode(stringToEncode, "utf-8").replace("+", "%20").replace("*", "%2A").replace("%7E", "~")>
		
		<cfreturn encodedString>
	</cffunction>
</cfcomponent>